/*
 * mppt_control.c
 *
 *  Created on: 2024629
 *      Author: psx
 */

#include "mppt_control.h"
#include "collect_Conversion.h"
#include "pwm.h"
#include "inflash.h"
#include "gpio.h"
#include "sl_protocol.h"
#include "task.h"
#include "uart_dev.h"
#include "parameter.h"


static void ConstantCurrentCharge(void);
static void ConstantVoltageCharge(void);
static void FloatingCharge(void);


/**
  * @brief  㶨ѹ
  * @param
  * @retval
  *
  */
void mppt_constantVoltage(float InVoltage)
{
    static float_t kp = 0.005;
    static float_t ki = 0.00001;

    float_t pv1Volt = g_otherParameter.Solar_In_Circuit_Voltage;
    float_t error = pv1Volt - InVoltage;
    float_t stepPwm = kp * error + ki * pv1Volt;
    g_controlParameter.dutyRatio += stepPwm;

    Set_duty_ratio(&g_controlParameter.dutyRatio);
}

/**
  * @brief  㶨ѹ()
  * @param
  * @retval
  *
  */
void mppt_constantVoltageB(float OutVoltage)
{
    static float_t kp = 0.005;
    static float_t ki = 0.00001;

    float_t outVolt = g_otherParameter.Battery_Voltage;

    float_t error = OutVoltage - outVolt;
    float_t stepPwm = kp * error + ki * outVolt;
    g_controlParameter.dutyRatio += stepPwm;

    Set_duty_ratio(&g_controlParameter.dutyRatio);
}

/**
  * @brief  㶨ѹˣ
  * @param
  * @retval
  *
  */
void mppt_constantVoltageO(float OutVoltage)
{
    static float_t kp = 0.005;
    static float_t ki = 0.00001;

    float_t outVolt = g_otherParameter.Output_Voltage;

    float_t error = OutVoltage - outVolt;
    float_t stepPwm = kp * error + ki * outVolt;
    g_controlParameter.dutyRatio += stepPwm;

    Set_duty_ratio(&g_controlParameter.dutyRatio);
}

/**
  * @brief  磨磩mpptʳ
  * @param
  * @retval
  *
  */
void ConstantCurrentCharge(void)
{
    mppt_constantVoltage(18);
}

/**
  * @brief  ѹ
  * @param
  * @retval
  *
  */
void ConstantVoltageCharge(void)
{
    mppt_constantVoltageO(g_controlParameter.constantVoltageChargeV);
}

/**
  * @brief  
  * @param
  * @retval
  *
  */
void FloatingCharge(void)
{
    mppt_constantVoltageO(g_controlParameter.FloatV);
}

/**
  * @brief  mpptģʽȷ
  * @param
  * @retval
  *
  */
void MpptContorlChoice(void)
{
    switch(g_otherParameter.MPPT_Mode) {

    case CONSTANTCURRENT:
        ConstantCurrentCharge();
        break;

    case CONSTANTVOLTAGE:
        ConstantVoltageCharge();
        break;

    case FLOAT:
        FloatingCharge();
        break;

    default:
        break;
    }
}

/**
  * @brief  mpptģʽȷ
  * @param
  * @retval
  *
  */
void MpptModeChoice(void)
{
    /* ̫ܰѹСһֵҳҲСһֵʱmpptֹͣ */
    if ((g_otherParameter.Input_Voltage < g_controlParameter.stopSolarOpenCircuitV
            && g_otherParameter.Charg_Current < 0.05) ){
//            && g_otherParameter.MPPT_Mode != NoWork) {
        g_otherParameter.MPPT_Mode = NoWork;

        TIM_Cmd(TIM3, DISABLE);
        TIM_SetCompare4(TIM4, 0);
        g_controlParameter.dutyRatio = 0;

        TimeSliceOffset_Register(&g_startMpptControl, Task_startMpptControl
                , startMpptControl_reloadVal, startMpptControl_offset);
        TimeSliceOffset_Unregister(&g_collectOpenCircuitVoltage);
        return;
    }

    /* صĵСһֵߵѹ߹Сø */
    if ((g_otherParameter.Charg_BatteryCurrent < 0.05 && g_otherParameter.Charg_BatteryCurrent > -0.05)
            || g_otherParameter.Battery_Voltage > 16 || g_otherParameter.Battery_Voltage < 8) {
        g_otherParameter.MPPT_Mode = FLOAT;
        g_otherParameter.batteryState = 0;
        return;
    }

    /*  */
    if ((g_controlParameter.constantVoltageV - 0.2) >= g_otherParameter.Battery_Voltage
            && g_otherParameter.Charg_Current > 0.1) {
        g_otherParameter.MPPT_Mode = CONSTANTCURRENT;
        return;
    }

    if ((g_controlParameter.constantVoltageV < g_otherParameter.Battery_Voltage)
//            && (g_controlParameter.floatI + 0.1 <= g_otherParameter.Charg_Current)) {
            && ((g_controlParameter.floatI + 0.1 <= g_otherParameter.Charg_BatteryCurrent) || (g_controlParameter.floatI + 0.1 <= -g_otherParameter.Charg_Current))) {
        g_otherParameter.MPPT_Mode = CONSTANTVOLTAGE;
        return;
    }

    if ((((g_controlParameter.constantVoltageV < g_otherParameter.Battery_Voltage)
            && (g_controlParameter.floatI > g_otherParameter.Charg_Current))
            && (g_controlParameter.floatI > g_otherParameter.Discharg_Current))) {
        g_otherParameter.MPPT_Mode = FLOAT;
        return;
    }

}

/**
  * @brief  mpptĿ
  * @param
  * @retval
  *
  */
void MpptContorl(void)
{
    g_otherParameter.Output_Voltage = get_PV_VOLT_OUT();
    g_otherParameter.Solar_In_Circuit_Voltage  = get_PV1_VOLT_IN();

//    /* adcɼȫΪ0,˳ж */
//    if (g_otherParameter.Discharg_Current == 0 && g_otherParameter.Charg_Current == 0) {
//        return;
//    }

    g_otherParameter.Charg_BatteryCurrent = g_otherParameter.Charg_Current - g_otherParameter.Discharg_Current;


    if (!g_otherParameter.overTemperature) {
        MpptModeChoice();
        MpptContorlChoice();
    }

}






